==> reverie_dreams-0.4.3%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/entity/misc/MagicBroomEntity.java <==
package cc.thonly.reverie_dreams.entity.misc;

import cc.thonly.polymer.entity.MagicBroomImpl;
import cc.thonly.polymer.entity.WheelChairImpl;
import cc.thonly.reverie_dreams.recipe.ItemStackWrapper;
import cc.thonly.reverie_dreams.server.PlayerInputManager;
import eu.pb4.polymer.core.api.entity.PolymerEntity;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1282;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1304;
import net.minecraft.class_1309;
import net.minecraft.class_1313;
import net.minecraft.class_1314;
import net.minecraft.class_1316;
import net.minecraft.class_1347;
import net.minecraft.class_1429;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_241;
import net.minecraft.class_243;
import net.minecraft.class_2945;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_5132;
import net.minecraft.class_5134;
import net.minecraft.class_5455;
import net.minecraft.class_8111;
import net.minecraft.entity.*;
import org.jetbrains.annotations.Nullable;

@Setter
@Getter
@ToString
public class MagicBroomEntity extends class_1314 implements class_1316 {
    public ItemStackWrapper itemWrapper = ItemStackWrapper.of(class_1802.field_8162.method_7854());
    public int damageTick = 0;
    public final int maxDamageTick = 20 * 8;
    public String ownerUUID = "";

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world) {
        super(entityType, world);
    }

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world, int x, int y, int z, ItemStackWrapper wrapper) {
        this(entityType, world);
        this.method_5814(x, y, z);
        this.itemWrapper = wrapper;
    }

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world, int x, int y, int z, ItemStackWrapper wrapper, String ownerUUID) {
        this(entityType, world, x, y, z, wrapper);
        this.ownerUUID = ownerUUID;
    }

    @Override
    protected void method_5693(class_2945.class_9222 builder) {
        super.method_5693(builder);
    }

    @Override
    protected void method_5959() {
        super.method_5959();
        this.field_6201.method_6277(0, new class_1347(this));
    }

    @Override
    public void method_5773() {
        super.method_5773();
        if (this.method_37908() instanceof class_3218 world) {
            this.method_5875(this.method_5782());
            if (!this.method_6059(class_1294.field_5905)) {
                this.method_6092(new class_1293(class_1294.field_5905, Integer.MAX_VALUE, 0, false, false));
            }
            if (!this.itemWrapper.isEmpty() && this.itemWrapper.getItemStack().isDamageable() && this.itemWrapper.getItemStack().getDamage() >= this.itemWrapper.getItemStack().getMaxDamage()) {
                this.method_64397(world, this.method_48923().method_48831(), Integer.MAX_VALUE);
            }
        }
    }

    @Override
    public void method_5837(class_3222 player) {
        super.method_5837(player);
        PolymerEntity polymerEntity = PolymerEntity.get(this);
        if (polymerEntity instanceof MagicBroomImpl impl) {
            impl.onTrackingStopped(player);
            impl.onCreated();
        }
    }

    @Override
    public void method_5742(class_3222 player) {
        super.method_5742(player);
        PolymerEntity polymerEntity = PolymerEntity.get(this);
        if (polymerEntity instanceof MagicBroomImpl impl) {
            impl.onTrackingStopped(player);
        }
    }

    @Override
    public boolean method_64397(class_3218 world, class_1282 source, float amount) {
        class_1297 attacker = source.method_5529();
        if (attacker != null && attacker.method_5715() && this.ownerUUID.intern().equalsIgnoreCase(attacker.method_5667().toString())) {
            if (!this.itemWrapper.isEmpty()) {
                class_1799 copiedStack = this.itemWrapper.getItemStack().copy();
                class_1542 itemEntity = new class_1542(world, this.method_23317(), this.method_23318(), this.method_23321(), copiedStack);
                world.method_8649(itemEntity);
                this.method_31472();
            }
        }
        return super.method_64397(world, source, amount);
    }

    @Override
    public void method_48922(class_1282 damageSource) {
        if (damageSource.method_49708(class_8111.field_42345)) {
            return;
        }
        super.method_48922(damageSource);
    }

    @Override
    protected void method_6074(class_3218 world, class_1282 source, float amount) {
        if (source.method_49708(class_8111.field_42345)) {
            return;
        }
        super.method_6074(world, source, amount);
    }

    @Override
    @Nullable
    public class_1309 method_5642() {
        class_1657 playerEntity;
        class_1297 entity;
        if ((entity = this.method_31483()) instanceof class_1657 && (playerEntity = (class_1657) entity).method_5805()) {
            return playerEntity;
        }
        return super.method_5642();
    }

    @Override
    protected void method_49481(class_1657 controllingPlayer, class_243 movementInput) {
        super.method_49481(controllingPlayer, movementInput);
        class_241 vec2f = this.getControlledRotation(controllingPlayer);
        this.method_5710(vec2f.field_1342, vec2f.field_1343);
        this.field_6283 = this.field_6241 = this.method_36454();
        this.field_6220 = this.field_6241;

        if (!this.itemWrapper.isEmpty() && !controllingPlayer.method_56992() && this.itemWrapper.getItemStack().isDamageable()) {
            this.damageTick++;
            if (this.damageTick > this.maxDamageTick) {
                this.itemWrapper.getItemStack().damage(1, this, class_1304.field_6173);
                this.damageTick = 0;
            }
        }

        if (controllingPlayer instanceof class_3222 player) {
            boolean keyLeft = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.LEFT);
            boolean keyRight = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.RIGHT);
            boolean keyForward = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.FORWARD);
            boolean keyBack = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.BACKWARD);
            boolean keySpeedUp = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.SPRINT);

            float strafe = keyLeft ? 0.5f : (keyRight ? -0.5f : 0);
            float vertical = keyForward ? -(player.method_36455() - 10) / 22.5f : 0;
            float forward = keyForward ? 3 : (keyBack ? -0.5f : 0);

            float speedMultiplier = keySpeedUp ? 1.8f : 1.0f;

            this.method_5724(0.245f * speedMultiplier, new class_243(strafe, vertical, forward));
            this.method_5784(class_1313.field_6308, this.method_18798());
            this.field_6007 = true;
        }
    }

    protected class_241 getControlledRotation(class_1309 controllingPassenger) {
        return new class_241(controllingPassenger.method_36455() * 0.5f, controllingPassenger.method_36454());
    }

    @Override
    protected class_243 method_49482(class_1657 controllingPlayer, class_243 movementInput) {
        float f = controllingPlayer.field_6212 * 0.5f;
        float g = controllingPlayer.field_6250;
        if (g <= 0.0f) {
            g *= 0.25f;
        }
        return new class_243(f, 0.0, g);
    }

    @Override
    public void method_6078(class_1282 damageSource) {
        super.method_6078(damageSource);
        class_1937 world = this.method_37908();
        if (this.itemWrapper.isEmpty()) {
            return;
        }
        class_1799 copiedStack = this.itemWrapper.getItemStack().copy();
        class_1542 itemEntity = new class_1542(world, this.method_23317(), this.method_23318(), this.method_23321(), copiedStack);
        world.method_8649(itemEntity);
    }

    @Override
    public class_1269 method_5992(class_1657 player, class_1268 hand) {
        class_1937 world = player.method_37908();
        if (!world.method_8608() && world instanceof class_3218 serverWorld) {

        }
        if (!this.method_5782() && !player.method_21823()) {
            if (!this.method_37908().field_9236) {
                player.method_5804(this);
            }
            return class_1269.field_5812;
        }
        return super.method_5992(player, hand);
    }

    public static class_5132 createAttributes() {
        return class_1429.method_61457()
                .method_26868(class_5134.field_23716, 15.0)
                .method_26868(class_5134.field_23719, 0.25)
                .method_26868(class_5134.field_23720, 0.15)
                .method_26868(class_5134.field_23718, 10.0)
                .method_26866();
    }

    @Override
    protected void method_5652(class_11372 view) {
        super.method_5652(view);
        view.method_71468("SummonedItem", ItemStackWrapper.CODEC, this.itemWrapper);
        view.method_71469("OwnerUUID", this.ownerUUID);
    }

    @Override
    protected void method_5749(class_11368 view) {
        super.method_5749(view);
        class_5455 registryManager = this.method_56673();
        this.itemWrapper = view.method_71426("SummonedItem", ItemStackWrapper.CODEC).orElse(ItemStackWrapper.of(class_1802.field_8162));

        this.ownerUUID = view.method_71428("OwnerUUID", "null");

    }

    @Override
    public void method_6154(int strength) {

    }

    @Override
    public boolean method_6153() {
        return true;
    }

    @Override
    public void method_6155(int height) {

    }

    @Override
    public void method_6156() {

    }
}

==> reverie_dreams-0.4.7%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/entity/misc/MagicBroomEntity.java <==
package cc.thonly.reverie_dreams.entity.misc;

import cc.thonly.polymer.entity.MagicBroomImpl;
import cc.thonly.reverie_dreams.recipe.ItemStackWrapper;
import cc.thonly.reverie_dreams.server.PlayerInputManager;
import cc.thonly.reverie_dreams.util.codec.UUIDCodec;
import eu.pb4.polymer.core.api.entity.PolymerEntity;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import net.minecraft.class_11368;
import net.minecraft.class_11372;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1282;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1304;
import net.minecraft.class_1309;
import net.minecraft.class_1313;
import net.minecraft.class_1314;
import net.minecraft.class_1316;
import net.minecraft.class_1347;
import net.minecraft.class_1429;
import net.minecraft.class_1542;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1937;
import net.minecraft.class_241;
import net.minecraft.class_243;
import net.minecraft.class_2945;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_5132;
import net.minecraft.class_5134;
import net.minecraft.class_5455;
import net.minecraft.class_8111;
import org.jetbrains.annotations.Nullable;

import java.util.UUID;

@Setter
@Getter
@ToString
public class MagicBroomEntity extends class_1314 implements class_1316 {
    public ItemStackWrapper itemWrapper = ItemStackWrapper.of(class_1802.field_8162.method_7854());
    public int damageTick = 0;
    public final int maxDamageTick = 20 * 8;
    @Nullable
    public UUID owner;

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world) {
        super(entityType, world);
    }

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world, int x, int y, int z, ItemStackWrapper wrapper) {
        this(entityType, world);
        this.method_5814(x, y, z);
        this.itemWrapper = wrapper;
    }

    public MagicBroomEntity(class_1299<? extends class_1314> entityType, class_1937 world, int x, int y, int z, ItemStackWrapper wrapper, UUID owner) {
        this(entityType, world, x, y, z, wrapper);
        this.owner = owner;
    }

    @Override
    protected void method_5693(class_2945.class_9222 builder) {
        super.method_5693(builder);
    }

    @Override
    protected void method_5959() {
        super.method_5959();
        this.field_6201.method_6277(0, new class_1347(this));
    }

    @Override
    public void method_5773() {
        super.method_5773();
        if (this.method_37908() instanceof class_3218 world) {
            this.method_5875(this.method_5782());
            if (!this.method_6059(class_1294.field_5905)) {
                this.method_6092(new class_1293(class_1294.field_5905, Integer.MAX_VALUE, 0, false, false));
            }
            if (!this.itemWrapper.isEmpty() && this.itemWrapper.getItemStack().isDamageableItem() && this.itemWrapper.getItemStack().getDamageValue() >= this.itemWrapper.getItemStack().getMaxDamage()) {
                this.method_64397(world, this.method_48923().method_48831(), Integer.MAX_VALUE);
            }
        }
    }

    @Override
    public void method_5837(class_3222 player) {
        super.method_5837(player);
        PolymerEntity polymerEntity = PolymerEntity.get(this);
        if (polymerEntity instanceof MagicBroomImpl impl) {
            impl.onTrackingStopped(player);
            impl.onCreated();
        }
    }

    @Override
    public void method_5742(class_3222 player) {
        super.method_5742(player);
        PolymerEntity polymerEntity = PolymerEntity.get(this);
        if (polymerEntity instanceof MagicBroomImpl impl) {
            impl.onTrackingStopped(player);
        }
    }

    @Override
    public boolean method_64397(class_3218 world, class_1282 source, float amount) {
        class_1297 attacker = source.method_5529();
        if (attacker != null && attacker.method_5715()) {
            if (!this.itemWrapper.isEmpty()) {
                class_1799 copiedStack = this.itemWrapper.getItemStack().copy();
                class_1542 itemEntity = new class_1542(world, this.method_23317(), this.method_23318(), this.method_23321(), copiedStack);
                world.method_8649(itemEntity);
                this.method_31472();
            }
        }
        return super.method_64397(world, source, amount);
    }

    @Override
    public void method_48922(class_1282 damageSource) {
        if (damageSource.method_49708(class_8111.field_42345)) {
            return;
        }
        super.method_48922(damageSource);
    }

    @Override
    protected void method_6074(class_3218 world, class_1282 source, float amount) {
        if (source.method_49708(class_8111.field_42345)) {
            return;
        }
        super.method_6074(world, source, amount);
    }

    @Override
    @Nullable
    public class_1309 method_5642() {
        class_1657 playerEntity;
        class_1297 entity;
        if ((entity = this.method_31483()) instanceof class_1657 && (playerEntity = (class_1657) entity).method_5805()) {
            return playerEntity;
        }
        return super.method_5642();
    }

    @Override
    protected void method_49481(class_1657 controllingPlayer, class_243 movementInput) {
        super.method_49481(controllingPlayer, movementInput);
        class_241 vec2f = this.getControlledRotation(controllingPlayer);
        this.method_5710(vec2f.field_1342, vec2f.field_1343);
        this.field_6283 = this.field_6241 = this.method_36454();
        this.field_6220 = this.field_6241;

        if (!this.itemWrapper.isEmpty() && !controllingPlayer.method_56992() && this.itemWrapper.getItemStack().isDamageableItem()) {
            this.damageTick++;
            if (this.damageTick > this.maxDamageTick) {
                this.itemWrapper.getItemStack().hurtAndBreak(1, this, class_1304.field_6173);
                this.damageTick = 0;
            }
        }

        if (controllingPlayer instanceof class_3222 player) {
            boolean keyLeft = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.LEFT);
            boolean keyRight = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.RIGHT);
            boolean keyForward = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.FORWARD);
            boolean keyBack = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.BACKWARD);
            boolean keySpeedUp = PlayerInputManager.isKeyDown(player, PlayerInputManager.InputKey.SPRINT);

            float strafe = keyLeft ? 0.5f : (keyRight ? -0.5f : 0);
            float vertical = keyForward ? -(player.method_36455() - 10) / 22.5f : 0;
            float forward = keyForward ? 3 : (keyBack ? -0.5f : 0);

            float speedMultiplier = 1.5f * (keySpeedUp ? 1.8f : 1.0f);

            this.method_5724(0.245f * speedMultiplier, new class_243(strafe, vertical, forward));
            this.method_5784(class_1313.field_6308, this.method_18798());
            this.field_6007 = true;
        }
    }

    protected class_241 getControlledRotation(class_1309 controllingPassenger) {
        return new class_241(controllingPassenger.method_36455() * 0.5f, controllingPassenger.method_36454());
    }

    @Override
    protected class_243 method_49482(class_1657 controllingPlayer, class_243 movementInput) {
        float f = controllingPlayer.field_6212 * 0.5f;
        float g = controllingPlayer.field_6250;
        if (g <= 0.0f) {
            g *= 0.25f;
        }
        return new class_243(f, 0.0, g);
    }

    @Override
    public void method_6078(class_1282 damageSource) {
        super.method_6078(damageSource);
        class_1937 world = this.method_37908();
        if (this.itemWrapper.isEmpty()) {
            return;
        }
        class_1799 copiedStack = this.itemWrapper.getItemStack().copy();
        class_1542 itemEntity = new class_1542(world, this.method_23317(), this.method_23318(), this.method_23321(), copiedStack);
        world.method_8649(itemEntity);
    }

    @Override
    public class_1269 method_5992(class_1657 player, class_1268 hand) {
        if (!this.method_5782() && !player.method_21823()) {
            if (!this.method_37908().field_9236) {
                player.method_5804(this);
            }
            return class_1269.field_5812;
        }
        return super.method_5992(player, hand);
    }

    public static class_5132 createAttributes() {
        return class_1429.method_61457()
                .method_26868(class_5134.field_23716, 100.0)
                .method_26868(class_5134.field_23719, 0.25)
                .method_26868(class_5134.field_23720, 0.15)
                .method_26868(class_5134.field_23718, 10.0)
                .method_26866();
    }

    @Override
    protected void method_5652(class_11372 view) {
        super.method_5652(view);
        view.method_71468("Item", ItemStackWrapper.CODEC, this.itemWrapper);
        if (this.owner != null) {
            view.method_71468("Owner", UUIDCodec.CODEC, this.owner);
        }
    }

    @Override
    protected void method_5749(class_11368 view) {
        super.method_5749(view);
        class_5455 registryManager = this.method_56673();
        this.itemWrapper = view.method_71426("Item", ItemStackWrapper.CODEC).orElse(ItemStackWrapper.of(class_1802.field_8162));
        view.method_71426("Owner", UUIDCodec.CODEC).ifPresent(value -> this.owner = value);

    }

    @Override
    public class_243 method_52538(class_1297 entity) {
        class_243 position = super.method_52538(entity);
        position = new class_243(position.field_1352, position.field_1351 - 1, position.field_1350);
        return position;
    }

    @Override
    public void method_6154(int strength) {

    }

    @Override
    public boolean method_6153() {
        return true;
    }

    @Override
    public void method_6155(int height) {

    }

    @Override
    public void method_6156() {

    }
}
